@@ -22,12 +22,19 @@
 #' @format NULL
 #' @docType class
 #'
-#' @section Methods:
+#' @section R6 Methods:
 #'
 #' - `$ToString()`: String representation of the DataType
 #' - `$Equals(other)`: Is the DataType equal to `other`
 #' - `$fields()`: The children fields associated with this type
+#' - `$code()`: Produces an R call of the data type.
 #'
+#' There are also some active bindings:
+#' - `$id`: integer Arrow type id.
+#' - `$name`: string Arrow type name.
+#' - `$num_fields`: number of child fields.
+#'
+#' @seealso [infer_type()]
 #' @rdname DataType
 #' @name DataType
 #' @seealso [`data-type`]
@@ -37,8 +44,8 @@ DataType <- R6Class("DataType",
     ToString = function() {
       DataType__ToString(self)
     },
-    Equals = function(other, ...) {
-      inherits(other, "DataType") && DataType__Equals(self, other)
+    Equals = function(other, check_metadata = FALSE, ...) {
+      inherits(other, "DataType") && DataType__Equals(self, other, isTRUE(check_metadata))
     },
     fields = function() {
       DataType__fields(self)
@@ -371,7 +378,7 @@ NestedType <- R6Class("NestedType", inherit = DataType)
 #' @param ... For `struct()`, a named list of types to define the struct columns
 #'
 #' @name data-type
-#' @return An Arrow type object inheriting from DataType.
+#' @return An Arrow type object inheriting from [DataType].
 #' @export
 #' @seealso [dictionary()] for creating a dictionary (factor-like) type.
 #' @examples
