@@ -17,17 +17,33 @@
 
 #include "./arrow_types.h"
 
-#if defined(ARROW_R_WITH_ARROW)
 #include <arrow/ipc/writer.h>
 #include <arrow/type.h>
 #include <arrow/util/key_value_metadata.h>
 
 // [[arrow::export]]
-std::shared_ptr<arrow::Schema> schema_(
+std::shared_ptr<arrow::Schema> Schema__from_fields(
     const std::vector<std::shared_ptr<arrow::Field>>& fields) {
   return arrow::schema(fields);
 }
 
+// [[arrow::export]]
+std::shared_ptr<arrow::Schema> Schema__from_list(cpp11::list field_list) {
+  int n = field_list.size();
+
+  bool nullable = true;
+  cpp11::strings names(field_list.attr(R_NamesSymbol));
+
+  std::vector<std::shared_ptr<arrow::Field>> fields(n);
+
+  for (int i = 0; i < n; i++) {
+    fields[i] = arrow::field(
+        names[i], cpp11::as_cpp<std::shared_ptr<arrow::DataType>>(field_list[i]),
+        nullable);
+  }
+  return arrow::schema(fields);
+}
+
 // [[arrow::export]]
 std::string Schema__ToString(const std::shared_ptr<arrow::Schema>& s) {
   return s->ToString();
@@ -48,6 +64,26 @@ std::shared_ptr<arrow::Field> Schema__field(const std::shared_ptr<arrow::Schema>
   return s->field(i);
 }
 
+// [[arrow::export]]
+std::shared_ptr<arrow::Schema> Schema__AddField(
+    const std::shared_ptr<arrow::Schema>& s, int i,
+    const std::shared_ptr<arrow::Field>& field) {
+  return ValueOrStop(s->AddField(i, field));
+}
+
+// [[arrow::export]]
+std::shared_ptr<arrow::Schema> Schema__SetField(
+    const std::shared_ptr<arrow::Schema>& s, int i,
+    const std::shared_ptr<arrow::Field>& field) {
+  return ValueOrStop(s->SetField(i, field));
+}
+
+// [[arrow::export]]
+std::shared_ptr<arrow::Schema> Schema__RemoveField(
+    const std::shared_ptr<arrow::Schema>& s, int i) {
+  return ValueOrStop(s->RemoveField(i));
+}
+
 // [[arrow::export]]
 std::shared_ptr<arrow::Field> Schema__GetFieldByName(
     const std::shared_ptr<arrow::Schema>& s, std::string x) {
@@ -93,14 +129,17 @@ cpp11::writable::list Schema__metadata(const std::shared_ptr<arrow::Schema>& sch
   return out;
 }
 
-// [[arrow::export]]
-std::shared_ptr<arrow::Schema> Schema__WithMetadata(
-    const std::shared_ptr<arrow::Schema>& schema, cpp11::strings metadata) {
+std::shared_ptr<arrow::KeyValueMetadata> strings_to_kvm(cpp11::strings metadata) {
   auto values = cpp11::as_cpp<std::vector<std::string>>(metadata);
   auto names = cpp11::as_cpp<std::vector<std::string>>(metadata.attr("names"));
 
-  auto kv =
-      std::make_shared<arrow::KeyValueMetadata>(std::move(names), std::move(values));
+  return std::make_shared<arrow::KeyValueMetadata>(std::move(names), std::move(values));
+}
+
+// [[arrow::export]]
+std::shared_ptr<arrow::Schema> Schema__WithMetadata(
+    const std::shared_ptr<arrow::Schema>& schema, cpp11::strings metadata) {
+  auto kv = strings_to_kvm(metadata);
   return schema->WithMetadata(std::move(kv));
 }
 
@@ -122,5 +161,3 @@ std::shared_ptr<arrow::Schema> arrow__UnifySchemas(
     const std::vector<std::shared_ptr<arrow::Schema>>& schemas) {
   return ValueOrStop(arrow::UnifySchemas(schemas));
 }
-
-#endif
