@@ -97,3 +97,27 @@ test_that("count returns an ungrouped tibble", {
     tbl
   )
 })
+
+test_that("tally raises appropriate error and message for names", {
+
+  expect_message(
+    tbl %>%
+      arrow_table() %>%
+      rename(n = some_grouping) %>%
+      group_by(n) %>%
+      tally() %>%
+      collect(),
+    regexp = 'Use `name = "new_name"` to pick a new name.',
+    fixed = TRUE
+  )
+
+  expect_error(
+    tbl %>%
+      arrow_table() %>%
+      group_by(some_grouping) %>%
+      tally(wt = int, name = 99) %>%
+      collect(),
+    "`name` must be a string or `NULL`.",
+    fixed = TRUE
+  )
+})
