@@ -15,16 +15,24 @@
 # specific language governing permissions and limitations
 # under the License.
 
-context("Field")
 
 test_that("field() factory", {
   x <- field("x", int32())
   expect_equal(x$type, int32())
   expect_equal(x$name, "x")
+  expect_true(x$nullable)
   expect_true(x == x)
   expect_false(x == field("x", int64()))
 })
 
+test_that("Field with nullable values", {
+  x <- field("x", int32(), nullable = FALSE)
+  expect_equal(x$type, int32())
+  expect_false(x$nullable)
+  expect_true(x == x)
+  expect_false(x == field("x", int32()))
+})
+
 test_that("Field validation", {
   expect_error(schema(b = 32), "b must be a DataType, not numeric")
 })
@@ -35,4 +43,24 @@ test_that("Print method for field", {
     print(field("zz", dictionary())),
     "Field\nzz: dictionary<values=string, indices=int32>"
   )
+
+  expect_output(
+    print(field("x", int32(), nullable = FALSE)),
+    "Field\nx: int32 not null"
+  )
+})
+
+test_that("Field to C-interface", {
+  field <- field("x", time32("s"))
+
+  # export the field via the C-interface
+  ptr <- allocate_arrow_schema()
+  field$export_to_c(ptr)
+
+  # then import it and check that the roundtripped value is the same
+  circle <- Field$import_from_c(ptr)
+  expect_equal(circle, field)
+
+  # must clean up the pointer or we leak
+  delete_arrow_schema(ptr)
 })
