@@ -18,6 +18,7 @@
 #include "arrow/compute/api_scalar.h"
 
 #include <memory>
+#include <optional>
 #include <sstream>
 #include <string>
 
@@ -332,7 +333,10 @@ static auto kRoundOptionsType = GetFunctionOptionsType<RoundOptions>(
 static auto kRoundTemporalOptionsType = GetFunctionOptionsType<RoundTemporalOptions>(
     DataMember("multiple", &RoundTemporalOptions::multiple),
     DataMember("unit", &RoundTemporalOptions::unit),
-    DataMember("week_starts_monday", &RoundTemporalOptions::week_starts_monday));
+    DataMember("week_starts_monday", &RoundTemporalOptions::week_starts_monday),
+    DataMember("ceil_is_strictly_greater",
+               &RoundTemporalOptions::ceil_is_strictly_greater),
+    DataMember("calendar_based_origin", &RoundTemporalOptions::calendar_based_origin));
 static auto kRoundToMultipleOptionsType = GetFunctionOptionsType<RoundToMultipleOptions>(
     DataMember("multiple", &RoundToMultipleOptions::multiple),
     DataMember("round_mode", &RoundToMultipleOptions::round_mode));
@@ -342,6 +346,11 @@ static auto kSetLookupOptionsType = GetFunctionOptionsType<SetLookupOptions>(
 static auto kSliceOptionsType = GetFunctionOptionsType<SliceOptions>(
     DataMember("start", &SliceOptions::start), DataMember("stop", &SliceOptions::stop),
     DataMember("step", &SliceOptions::step));
+static auto kListSliceOptionsType = GetFunctionOptionsType<ListSliceOptions>(
+    DataMember("start", &ListSliceOptions::start),
+    DataMember("stop", &ListSliceOptions::stop),
+    DataMember("step", &ListSliceOptions::step),
+    DataMember("return_fixed_size_list", &ListSliceOptions::return_fixed_size_list));
 static auto kSplitPatternOptionsType = GetFunctionOptionsType<SplitPatternOptions>(
     DataMember("pattern", &SplitPatternOptions::pattern),
     DataMember("max_splits", &SplitPatternOptions::max_splits),
@@ -356,7 +365,7 @@ static auto kStrptimeOptionsType = GetFunctionOptionsType<StrptimeOptions>(
     DataMember("unit", &StrptimeOptions::unit),
     DataMember("error_is_null", &StrptimeOptions::error_is_null));
 static auto kStructFieldOptionsType = GetFunctionOptionsType<StructFieldOptions>(
-    DataMember("indices", &StructFieldOptions::indices));
+    DataMember("field_ref", &StructFieldOptions::field_ref));
 static auto kTrimOptionsType = GetFunctionOptionsType<TrimOptions>(
     DataMember("characters", &TrimOptions::characters));
 static auto kUtf8NormalizeOptionsType = GetFunctionOptionsType<Utf8NormalizeOptions>(
@@ -366,7 +375,6 @@ static auto kWeekOptionsType = GetFunctionOptionsType<WeekOptions>(
     DataMember("count_from_zero", &WeekOptions::count_from_zero),
     DataMember("first_week_is_fully_in_year", &WeekOptions::first_week_is_fully_in_year));
 static auto kRandomOptionsType = GetFunctionOptionsType<RandomOptions>(
-    DataMember("length", &RandomOptions::length),
     DataMember("initializer", &RandomOptions::initializer),
     DataMember("seed", &RandomOptions::seed));
 
@@ -491,11 +499,15 @@ RoundOptions::RoundOptions(int64_t ndigits, RoundMode round_mode)
 constexpr char RoundOptions::kTypeName[];
 
 RoundTemporalOptions::RoundTemporalOptions(int multiple, CalendarUnit unit,
-                                           bool week_starts_monday)
+                                           bool week_starts_monday,
+                                           bool ceil_is_strictly_greater,
+                                           bool calendar_based_origin)
     : FunctionOptions(internal::kRoundTemporalOptionsType),
       multiple(std::move(multiple)),
       unit(unit),
-      week_starts_monday(week_starts_monday) {}
+      week_starts_monday(week_starts_monday),
+      ceil_is_strictly_greater(ceil_is_strictly_greater),
+      calendar_based_origin(calendar_based_origin) {}
 constexpr char RoundTemporalOptions::kTypeName[];
 
 RoundToMultipleOptions::RoundToMultipleOptions(double multiple, RoundMode round_mode)
@@ -522,6 +534,17 @@ SliceOptions::SliceOptions(int64_t start, int64_t stop, int64_t step)
 SliceOptions::SliceOptions() : SliceOptions(0, 0, 1) {}
 constexpr char SliceOptions::kTypeName[];
 
+ListSliceOptions::ListSliceOptions(int64_t start, std::optional<int64_t> stop,
+                                   int64_t step,
+                                   std::optional<bool> return_fixed_size_list)
+    : FunctionOptions(internal::kListSliceOptionsType),
+      start(start),
+      stop(stop),
+      step(step),
+      return_fixed_size_list(return_fixed_size_list) {}
+ListSliceOptions::ListSliceOptions() : ListSliceOptions(0) {}
+constexpr char ListSliceOptions::kTypeName[];
+
 SplitOptions::SplitOptions(int64_t max_splits, bool reverse)
     : FunctionOptions(internal::kSplitOptionsType),
       max_splits(max_splits),
@@ -555,8 +578,13 @@ StrptimeOptions::StrptimeOptions() : StrptimeOptions("", TimeUnit::MICRO, false)
 constexpr char StrptimeOptions::kTypeName[];
 
 StructFieldOptions::StructFieldOptions(std::vector<int> indices)
-    : FunctionOptions(internal::kStructFieldOptionsType), indices(std::move(indices)) {}
-StructFieldOptions::StructFieldOptions() : StructFieldOptions(std::vector<int>()) {}
+    : FunctionOptions(internal::kStructFieldOptionsType), field_ref(std::move(indices)) {}
+StructFieldOptions::StructFieldOptions(std::initializer_list<int> indices)
+    : FunctionOptions(internal::kStructFieldOptionsType), field_ref(std::move(indices)) {}
+StructFieldOptions::StructFieldOptions(FieldRef ref)
+    : FunctionOptions(internal::kStructFieldOptionsType), field_ref(std::move(ref)) {}
+StructFieldOptions::StructFieldOptions()
+    : FunctionOptions(internal::kStructFieldOptionsType) {}
 constexpr char StructFieldOptions::kTypeName[];
 
 TrimOptions::TrimOptions(std::string characters)
@@ -576,12 +604,11 @@ WeekOptions::WeekOptions(bool week_starts_monday, bool count_from_zero,
       first_week_is_fully_in_year(first_week_is_fully_in_year) {}
 constexpr char WeekOptions::kTypeName[];
 
-RandomOptions::RandomOptions(int64_t length, Initializer initializer, uint64_t seed)
+RandomOptions::RandomOptions(Initializer initializer, uint64_t seed)
     : FunctionOptions(internal::kRandomOptionsType),
-      length(length),
       initializer(initializer),
       seed(seed) {}
-RandomOptions::RandomOptions() : RandomOptions(0, SystemRandom, 0) {}
+RandomOptions::RandomOptions() : RandomOptions(SystemRandom, 0) {}
 constexpr char RandomOptions::kTypeName[];
 
 namespace internal {
@@ -592,6 +619,7 @@ void RegisterScalarOptions(FunctionRegistry* registry) {
   DCHECK_OK(registry->AddFunctionOptionsType(kElementWiseAggregateOptionsType));
   DCHECK_OK(registry->AddFunctionOptionsType(kExtractRegexOptionsType));
   DCHECK_OK(registry->AddFunctionOptionsType(kJoinOptionsType));
+  DCHECK_OK(registry->AddFunctionOptionsType(kListSliceOptionsType));
   DCHECK_OK(registry->AddFunctionOptionsType(kMakeStructOptionsType));
   DCHECK_OK(registry->AddFunctionOptionsType(kMapLookupOptionsType));
   DCHECK_OK(registry->AddFunctionOptionsType(kMatchSubstringOptionsType));
@@ -648,6 +676,7 @@ SCALAR_ARITHMETIC_UNARY(Negate, "negate", "negate_checked")
 SCALAR_ARITHMETIC_UNARY(Sin, "sin", "sin_checked")
 SCALAR_ARITHMETIC_UNARY(Tan, "tan", "tan_checked")
 SCALAR_EAGER_UNARY(Atan, "atan")
+SCALAR_EAGER_UNARY(Exp, "exp")
 SCALAR_EAGER_UNARY(Sign, "sign")
 
 Result<Datum> Round(const Datum& arg, RoundOptions options, ExecContext* ctx) {
@@ -833,6 +862,20 @@ Result<Datum> Week(const Datum& arg, WeekOptions options, ExecContext* ctx) {
   return CallFunction("week", {arg}, &options, ctx);
 }
 
+SCALAR_EAGER_BINARY(YearsBetween, "years_between")
+SCALAR_EAGER_BINARY(QuartersBetween, "quarters_between")
+SCALAR_EAGER_BINARY(MonthsBetween, "month_interval_between")
+SCALAR_EAGER_BINARY(WeeksBetween, "weeks_between")
+SCALAR_EAGER_BINARY(MonthDayNanoBetween, "month_day_nano_interval_between")
+SCALAR_EAGER_BINARY(DayTimeBetween, "day_time_interval_between")
+SCALAR_EAGER_BINARY(DaysBetween, "days_between")
+SCALAR_EAGER_BINARY(HoursBetween, "hours_between")
+SCALAR_EAGER_BINARY(MinutesBetween, "minutes_between")
+SCALAR_EAGER_BINARY(SecondsBetween, "seconds_between")
+SCALAR_EAGER_BINARY(MillisecondsBetween, "milliseconds_between")
+SCALAR_EAGER_BINARY(MicrosecondsBetween, "microseconds_between")
+SCALAR_EAGER_BINARY(NanosecondsBetween, "nanoseconds_between")
+
 // ----------------------------------------------------------------------
 // Structural transforms
 Result<Datum> MapLookup(const Datum& arg, MapLookupOptions options, ExecContext* ctx) {
