@@ -68,6 +68,31 @@ struct NumericToStringCastFunctor {
   }
 };
 
+template <typename O, typename I>
+struct DecimalToStringCastFunctor {
+  using value_type = typename TypeTraits<I>::CType;
+  using BuilderType = typename TypeTraits<O>::BuilderType;
+  using FormatterType = StringFormatter<I>;
+
+  static Status Exec(KernelContext* ctx, const ExecSpan& batch, ExecResult* out) {
+    const ArraySpan& input = batch[0].array;
+    FormatterType formatter(input.type);
+    BuilderType builder(input.type->GetSharedPtr(), ctx->memory_pool());
+    RETURN_NOT_OK(VisitArraySpanInline<I>(
+        input,
+        [&](std::string_view bytes) {
+          value_type value(reinterpret_cast<const uint8_t*>(bytes.data()));
+          return formatter(value, [&](std::string_view v) { return builder.Append(v); });
+        },
+        [&]() { return builder.AppendNull(); }));
+
+    std::shared_ptr<Array> output_array;
+    RETURN_NOT_OK(builder.Finish(&output_array));
+    out->value = std::move(output_array->data());
+    return Status::OK();
+  }
+};
+
 // ----------------------------------------------------------------------
 // Temporal to String
 
@@ -385,14 +410,27 @@ void AddNumberToStringCasts(CastFunction* func) {
   }
 }
 
+template <typename OutType>
+void AddDecimalToStringCasts(CastFunction* func) {
+  auto out_ty = TypeTraits<OutType>::type_singleton();
+  for (const auto& in_tid : std::vector<Type::type>{Type::DECIMAL128, Type::DECIMAL256}) {
+    DCHECK_OK(
+        func->AddKernel(in_tid, {in_tid}, out_ty,
+                        GenerateDecimal<DecimalToStringCastFunctor, OutType>(in_tid),
+                        NullHandling::COMPUTED_NO_PREALLOCATE));
+  }
+}
+
 template <typename OutType>
 void AddTemporalToStringCasts(CastFunction* func) {
   auto out_ty = TypeTraits<OutType>::type_singleton();
-  for (const std::shared_ptr<DataType>& in_ty : TemporalTypes()) {
-    DCHECK_OK(
-        func->AddKernel(in_ty->id(), {InputType(in_ty->id())}, out_ty,
-                        GenerateTemporal<TemporalToStringCastFunctor, OutType>(*in_ty),
-                        NullHandling::COMPUTED_NO_PREALLOCATE));
+  for (const auto& types : {TemporalTypes(), DurationTypes()}) {
+    for (const std::shared_ptr<DataType>& in_ty : types) {
+      DCHECK_OK(
+          func->AddKernel(in_ty->id(), {InputType(in_ty->id())}, out_ty,
+                          GenerateTemporal<TemporalToStringCastFunctor, OutType>(*in_ty),
+                          NullHandling::COMPUTED_NO_PREALLOCATE));
+    }
   }
 }
 
@@ -429,6 +467,7 @@ std::vector<std::shared_ptr<CastFunction>> GetBinaryLikeCasts() {
   auto cast_string = std::make_shared<CastFunction>("cast_string", Type::STRING);
   AddCommonCasts(Type::STRING, utf8(), cast_string.get());
   AddNumberToStringCasts<StringType>(cast_string.get());
+  AddDecimalToStringCasts<StringType>(cast_string.get());
   AddTemporalToStringCasts<StringType>(cast_string.get());
   AddBinaryToBinaryCast<StringType>(cast_string.get());
 
@@ -436,6 +475,7 @@ std::vector<std::shared_ptr<CastFunction>> GetBinaryLikeCasts() {
       std::make_shared<CastFunction>("cast_large_string", Type::LARGE_STRING);
   AddCommonCasts(Type::LARGE_STRING, large_utf8(), cast_large_string.get());
   AddNumberToStringCasts<LargeStringType>(cast_large_string.get());
+  AddDecimalToStringCasts<LargeStringType>(cast_large_string.get());
   AddTemporalToStringCasts<LargeStringType>(cast_large_string.get());
   AddBinaryToBinaryCast<LargeStringType>(cast_large_string.get());
 
