@@ -38,6 +38,7 @@
 #include "arrow/result.h"
 #include "arrow/type.h"
 #include "arrow/util/async_generator.h"
+#include "arrow/util/bit_util.h"
 #include "arrow/util/iterator.h"
 #include "arrow/util/logging.h"
 #include "arrow/util/tracing_internal.h"
@@ -52,9 +53,98 @@ using internal::SerialExecutor;
 
 namespace dataset {
 
+struct CsvInspectedFragment : public InspectedFragment {
+  CsvInspectedFragment(std::vector<std::string> column_names,
+                       std::shared_ptr<io::InputStream> input_stream, int64_t num_bytes)
+      : InspectedFragment(std::move(column_names)),
+        input_stream(std::move(input_stream)),
+        num_bytes(num_bytes) {}
+  // We need to start reading the file in order to figure out the column names and
+  // so we save off the input stream
+  std::shared_ptr<io::InputStream> input_stream;
+  int64_t num_bytes;
+};
+
+class CsvFileScanner : public FragmentScanner {
+ public:
+  CsvFileScanner(std::shared_ptr<csv::StreamingReader> reader, int num_batches,
+                 int64_t best_guess_bytes_per_batch)
+      : reader_(std::move(reader)),
+        num_batches_(num_batches),
+        best_guess_bytes_per_batch_(best_guess_bytes_per_batch) {}
+
+  Future<std::shared_ptr<RecordBatch>> ScanBatch(int batch_number) override {
+    // This should be called in increasing order but let's verify that in case it changes.
+    // It would be easy enough to handle out of order but no need for that complexity at
+    // the moment.
+    DCHECK_EQ(scanned_so_far_++, batch_number);
+    return reader_->ReadNextAsync();
+  }
+
+  int64_t EstimatedDataBytes(int batch_number) override {
+    return best_guess_bytes_per_batch_;
+  }
+
+  int NumBatches() override { return num_batches_; }
+
+  static Result<csv::ConvertOptions> GetConvertOptions(
+      const CsvFragmentScanOptions& csv_options, const FragmentScanRequest& scan_request,
+      const CsvInspectedFragment& inspected_fragment) {
+    // We use the convert options given from the user but override which columns we are
+    // looking for.
+    auto convert_options = csv_options.convert_options;
+    std::vector<std::string> columns;
+    std::unordered_map<std::string, std::shared_ptr<DataType>> column_types;
+    for (const auto& scan_column : scan_request.columns) {
+      if (scan_column.path.indices().size() != 1) {
+        return Status::Invalid("CSV reader does not supported nested references");
+      }
+      const std::string& column_name =
+          inspected_fragment.column_names[scan_column.path.indices()[0]];
+      columns.push_back(column_name);
+      column_types[column_name] = scan_column.requested_type->GetSharedPtr();
+    }
+    convert_options.include_columns = std::move(columns);
+    convert_options.column_types = std::move(column_types);
+    return std::move(convert_options);
+  }
+
+  static Future<std::shared_ptr<FragmentScanner>> Make(
+      const CsvFragmentScanOptions& csv_options, const FragmentScanRequest& scan_request,
+      const CsvInspectedFragment& inspected_fragment, Executor* cpu_executor) {
+    auto read_options = csv_options.read_options;
+
+    int num_batches = static_cast<int>(bit_util::CeilDiv(
+        inspected_fragment.num_bytes, static_cast<int64_t>(read_options.block_size)));
+    // Could be better, but a reasonable starting point.  CSV presumably takes up more
+    // space than an in-memory format so this should be conservative.
+    int64_t best_guess_bytes_per_batch = read_options.block_size;
+    ARROW_ASSIGN_OR_RAISE(
+        csv::ConvertOptions convert_options,
+        GetConvertOptions(csv_options, scan_request, inspected_fragment));
+
+    return csv::StreamingReader::MakeAsync(
+               io::default_io_context(), inspected_fragment.input_stream, cpu_executor,
+               read_options, csv_options.parse_options, convert_options)
+        .Then([num_batches, best_guess_bytes_per_batch](
+                  const std::shared_ptr<csv::StreamingReader>& reader)
+                  -> std::shared_ptr<FragmentScanner> {
+          return std::make_shared<CsvFileScanner>(reader, num_batches,
+                                                  best_guess_bytes_per_batch);
+        });
+  }
+
+ private:
+  std::shared_ptr<csv::StreamingReader> reader_;
+  int num_batches_;
+  int64_t best_guess_bytes_per_batch_;
+
+  int scanned_so_far_ = 0;
+};
+
 using RecordBatchGenerator = std::function<Future<std::shared_ptr<RecordBatch>>()>;
 
-Result<std::unordered_set<std::string>> GetColumnNames(
+Result<std::vector<std::string>> GetOrderedColumnNames(
     const csv::ReadOptions& read_options, const csv::ParseOptions& parse_options,
     std::string_view first_block, MemoryPool* pool) {
   // Skip BOM when reading column names (ARROW-14644, ARROW-17382)
@@ -64,13 +154,7 @@ Result<std::unordered_set<std::string>> GetColumnNames(
   size = size - static_cast<uint32_t>(data_no_bom - data);
   first_block = std::string_view(reinterpret_cast<const char*>(data_no_bom), size);
   if (!read_options.column_names.empty()) {
-    std::unordered_set<std::string> column_names;
-    for (const auto& s : read_options.column_names) {
-      if (!column_names.emplace(s).second) {
-        return Status::Invalid("CSV file contained multiple columns named ", s);
-      }
-    }
-    return column_names;
+    return read_options.column_names;
   }
 
   uint32_t parsed_size = 0;
@@ -90,14 +174,14 @@ Result<std::unordered_set<std::string>> GetColumnNames(
     return Status::Invalid("No columns in CSV file");
   }
 
-  std::unordered_set<std::string> column_names;
+  std::vector<std::string> column_names;
 
   if (read_options.autogenerate_column_names) {
     column_names.reserve(parser.num_cols());
     for (int32_t i = 0; i < parser.num_cols(); ++i) {
       std::stringstream ss;
       ss << "f" << i;
-      column_names.emplace(ss.str());
+      column_names.emplace_back(ss.str());
     }
     return column_names;
   }
@@ -105,15 +189,28 @@ Result<std::unordered_set<std::string>> GetColumnNames(
   RETURN_NOT_OK(
       parser.VisitLastRow([&](const uint8_t* data, uint32_t size, bool quoted) -> Status {
         std::string_view view{reinterpret_cast<const char*>(data), size};
-        if (column_names.emplace(std::string(view)).second) {
-          return Status::OK();
-        }
-        return Status::Invalid("CSV file contained multiple columns named ", view);
+        column_names.emplace_back(view);
+        return Status::OK();
       }));
 
   return column_names;
 }
 
+Result<std::unordered_set<std::string>> GetColumnNames(
+    const csv::ReadOptions& read_options, const csv::ParseOptions& parse_options,
+    std::string_view first_block, MemoryPool* pool) {
+  ARROW_ASSIGN_OR_RAISE(
+      std::vector<std::string> ordered_names,
+      GetOrderedColumnNames(read_options, parse_options, first_block, pool));
+  std::unordered_set<std::string> unordered_names;
+  for (const auto& column : ordered_names) {
+    if (!unordered_names.emplace(column).second) {
+      return Status::Invalid("CSV file contained multiple columns named ", column);
+    }
+  }
+  return unordered_names;
+}
+
 static inline Result<csv::ConvertOptions> GetConvertOptions(
     const CsvFileFormat& format, const ScanOptions* scan_options,
     const std::string_view first_block) {
@@ -251,6 +348,8 @@ static RecordBatchGenerator GeneratorFromReader(
   return MakeFromFuture(std::move(gen_fut));
 }
 
+CsvFileFormat::CsvFileFormat() : FileFormat(std::make_shared<CsvFragmentScanOptions>()) {}
+
 bool CsvFileFormat::Equals(const FileFormat& format) const {
   if (type_name() != format.type_name()) return false;
 
@@ -312,6 +411,52 @@ Future<std::optional<int64_t>> CsvFileFormat::CountRows(
       .Then([](int64_t count) { return std::make_optional<int64_t>(count); });
 }
 
+Future<std::shared_ptr<FragmentScanner>> CsvFileFormat::BeginScan(
+    const FragmentScanRequest& request, const InspectedFragment& inspected_fragment,
+    const FragmentScanOptions* format_options, compute::ExecContext* exec_context) const {
+  auto csv_options = static_cast<const CsvFragmentScanOptions*>(format_options);
+  auto csv_fragment = static_cast<const CsvInspectedFragment&>(inspected_fragment);
+  return CsvFileScanner::Make(*csv_options, request, csv_fragment,
+                              exec_context->executor());
+}
+
+Result<std::shared_ptr<InspectedFragment>> DoInspectFragment(
+    const FileSource& source, const CsvFragmentScanOptions& csv_options,
+    compute::ExecContext* exec_context) {
+  ARROW_ASSIGN_OR_RAISE(auto input, source.OpenCompressed());
+  if (csv_options.stream_transform_func) {
+    ARROW_ASSIGN_OR_RAISE(input, csv_options.stream_transform_func(input));
+  }
+  ARROW_ASSIGN_OR_RAISE(
+      input, io::BufferedInputStream::Create(csv_options.read_options.block_size,
+                                             default_memory_pool(), std::move(input)));
+
+  ARROW_ASSIGN_OR_RAISE(std::string_view first_block,
+                        input->Peek(csv_options.read_options.block_size));
+
+  ARROW_ASSIGN_OR_RAISE(
+      std::vector<std::string> column_names,
+      GetOrderedColumnNames(csv_options.read_options, csv_options.parse_options,
+                            first_block, exec_context->memory_pool()));
+  return std::make_shared<CsvInspectedFragment>(std::move(column_names), std::move(input),
+                                                source.Size());
+}
+
+Future<std::shared_ptr<InspectedFragment>> CsvFileFormat::InspectFragment(
+    const FileSource& source, const FragmentScanOptions* format_options,
+    compute::ExecContext* exec_context) const {
+  auto csv_options = static_cast<const CsvFragmentScanOptions*>(format_options);
+  Executor* io_executor;
+  if (source.filesystem()) {
+    io_executor = source.filesystem()->io_context().executor();
+  } else {
+    io_executor = exec_context->executor();
+  }
+  return DeferNotOk(io_executor->Submit([source, csv_options, exec_context]() {
+    return DoInspectFragment(source, *csv_options, exec_context);
+  }));
+}
+
 //
 // CsvFileWriter, CsvFileWriteOptions
 //
