@@ -21,14 +21,16 @@
 #pragma once
 
 #include <memory>
+#include <optional>
 #include <string>
+#include <string_view>
 #include <unordered_map>
 
 #include "arrow/flight/server.h"
 #include "arrow/flight/sql/server.h"
 #include "arrow/flight/sql/types.h"
 #include "arrow/flight/sql/visibility.h"
-#include "arrow/util/optional.h"
+#include "arrow/flight/types.h"
 
 namespace arrow {
 namespace flight {
@@ -43,18 +45,32 @@ namespace sql {
 struct ARROW_FLIGHT_SQL_EXPORT StatementQuery {
   /// \brief The SQL query.
   std::string query;
+  /// \brief The transaction ID, if specified (else a blank string).
+  std::string transaction_id;
+};
+
+/// \brief A Substrait plan to execute.
+struct ARROW_FLIGHT_SQL_EXPORT StatementSubstraitPlan {
+  /// \brief The Substrait plan.
+  SubstraitPlan plan;
+  /// \brief The transaction ID, if specified (else a blank string).
+  std::string transaction_id;
 };
 
 /// \brief A SQL update query.
 struct ARROW_FLIGHT_SQL_EXPORT StatementUpdate {
   /// \brief The SQL query.
   std::string query;
+  /// \brief The transaction ID, if specified (else a blank string).
+  std::string transaction_id;
 };
 
 /// \brief A request to execute a query.
 struct ARROW_FLIGHT_SQL_EXPORT StatementQueryTicket {
   /// \brief The server-generated opaque identifier for the query.
   std::string statement_handle;
+
+  static arrow::Result<StatementQueryTicket> Deserialize(std::string_view serialized);
 };
 
 /// \brief A prepared query statement.
@@ -78,19 +94,19 @@ struct ARROW_FLIGHT_SQL_EXPORT GetSqlInfo {
 /// \brief A request to list database schemas.
 struct ARROW_FLIGHT_SQL_EXPORT GetDbSchemas {
   /// \brief An optional database catalog to filter on.
-  util::optional<std::string> catalog;
+  std::optional<std::string> catalog;
   /// \brief An optional database schema to filter on.
-  util::optional<std::string> db_schema_filter_pattern;
+  std::optional<std::string> db_schema_filter_pattern;
 };
 
 /// \brief A request to list database tables.
 struct ARROW_FLIGHT_SQL_EXPORT GetTables {
   /// \brief An optional database catalog to filter on.
-  util::optional<std::string> catalog;
+  std::optional<std::string> catalog;
   /// \brief An optional database schema to filter on.
-  util::optional<std::string> db_schema_filter_pattern;
+  std::optional<std::string> db_schema_filter_pattern;
   /// \brief An optional table name to filter on.
-  util::optional<std::string> table_name_filter_pattern;
+  std::optional<std::string> table_name_filter_pattern;
   /// \brief A list of table types to filter on.
   std::vector<std::string> table_types;
   /// \brief Whether to include the Arrow schema in the response.
@@ -100,7 +116,7 @@ struct ARROW_FLIGHT_SQL_EXPORT GetTables {
 /// \brief A request to get SQL data type information.
 struct ARROW_FLIGHT_SQL_EXPORT GetXdbcTypeInfo {
   /// \brief A specific SQL type ID to fetch information about.
-  util::optional<int> data_type;
+  std::optional<int> data_type;
 };
 
 /// \brief A request to list primary keys of a table.
@@ -131,10 +147,66 @@ struct ARROW_FLIGHT_SQL_EXPORT GetCrossReference {
   TableRef fk_table_ref;
 };
 
+/// \brief A request to start a new transaction.
+struct ARROW_FLIGHT_SQL_EXPORT ActionBeginTransactionRequest {};
+
+/// \brief A request to create a new savepoint.
+struct ARROW_FLIGHT_SQL_EXPORT ActionBeginSavepointRequest {
+  std::string transaction_id;
+  std::string name;
+};
+
+/// \brief The result of starting a new savepoint.
+struct ARROW_FLIGHT_SQL_EXPORT ActionBeginSavepointResult {
+  std::string savepoint_id;
+};
+
+/// \brief The result of starting a new transaction.
+struct ARROW_FLIGHT_SQL_EXPORT ActionBeginTransactionResult {
+  std::string transaction_id;
+};
+
+/// \brief A request to end a savepoint.
+struct ARROW_FLIGHT_SQL_EXPORT ActionEndSavepointRequest {
+  enum EndSavepoint {
+    kRelease,
+    kRollback,
+  };
+
+  std::string savepoint_id;
+  EndSavepoint action;
+};
+
+/// \brief A request to end a transaction.
+struct ARROW_FLIGHT_SQL_EXPORT ActionEndTransactionRequest {
+  enum EndTransaction {
+    kCommit,
+    kRollback,
+  };
+
+  std::string transaction_id;
+  EndTransaction action;
+};
+
+/// \brief An explicit request to cancel a running query.
+struct ARROW_FLIGHT_SQL_EXPORT ActionCancelQueryRequest {
+  std::unique_ptr<FlightInfo> info;
+};
+
 /// \brief A request to create a new prepared statement.
 struct ARROW_FLIGHT_SQL_EXPORT ActionCreatePreparedStatementRequest {
   /// \brief The SQL query.
   std::string query;
+  /// \brief The transaction ID, if specified (else a blank string).
+  std::string transaction_id;
+};
+
+/// \brief A request to create a new prepared statement with a Substrait plan.
+struct ARROW_FLIGHT_SQL_EXPORT ActionCreatePreparedSubstraitPlanRequest {
+  /// \brief The serialized Substrait plan.
+  SubstraitPlan plan;
+  /// \brief The transaction ID, if specified (else a blank string).
+  std::string transaction_id;
 };
 
 /// \brief A request to close a prepared statement.
@@ -188,6 +260,15 @@ class ARROW_FLIGHT_SQL_EXPORT FlightSqlServerBase : public FlightServerBase {
       const ServerCallContext& context, const StatementQuery& command,
       const FlightDescriptor& descriptor);
 
+  /// \brief Get a FlightInfo for executing a Substrait plan.
+  /// \param[in] context      Per-call context.
+  /// \param[in] command      The StatementSubstraitPlan object containing the plan.
+  /// \param[in] descriptor   The descriptor identifying the data stream.
+  /// \return                 The FlightInfo describing where to access the dataset.
+  virtual arrow::Result<std::unique_ptr<FlightInfo>> GetFlightInfoSubstraitPlan(
+      const ServerCallContext& context, const StatementSubstraitPlan& command,
+      const FlightDescriptor& descriptor);
+
   /// \brief Get a FlightDataStream containing the query results.
   /// \param[in] context      Per-call context.
   /// \param[in] command      The StatementQueryTicket containing the statement handle.
@@ -221,6 +302,34 @@ class ARROW_FLIGHT_SQL_EXPORT FlightSqlServerBase : public FlightServerBase {
   virtual arrow::Result<std::unique_ptr<FlightInfo>> GetFlightInfoCatalogs(
       const ServerCallContext& context, const FlightDescriptor& descriptor);
 
+  /// \brief Get the schema of the result set of a query.
+  /// \param[in] context      Per-call context.
+  /// \param[in] command      The StatementQuery containing the SQL query.
+  /// \param[in] descriptor   The descriptor identifying the data stream.
+  /// \return                 The schema of the result set.
+  virtual arrow::Result<std::unique_ptr<SchemaResult>> GetSchemaStatement(
+      const ServerCallContext& context, const StatementQuery& command,
+      const FlightDescriptor& descriptor);
+
+  /// \brief Get the schema of the result set of a Substrait plan.
+  /// \param[in] context      Per-call context.
+  /// \param[in] command      The StatementQuery containing the plan.
+  /// \param[in] descriptor   The descriptor identifying the data stream.
+  /// \return                 The schema of the result set.
+  virtual arrow::Result<std::unique_ptr<SchemaResult>> GetSchemaSubstraitPlan(
+      const ServerCallContext& context, const StatementSubstraitPlan& command,
+      const FlightDescriptor& descriptor);
+
+  /// \brief Get the schema of the result set of a prepared statement.
+  /// \param[in] context      Per-call context.
+  /// \param[in] command      The PreparedStatementQuery containing the
+  ///                         prepared statement handle.
+  /// \param[in] descriptor   The descriptor identifying the data stream.
+  /// \return                 The schema of the result set.
+  virtual arrow::Result<std::unique_ptr<SchemaResult>> GetSchemaPreparedStatement(
+      const ServerCallContext& context, const PreparedStatementQuery& command,
+      const FlightDescriptor& descriptor);
+
   /// \brief Get a FlightDataStream containing the list of catalogs.
   /// \param[in] context  Per-call context.
   /// \return             An interface for sending data back to the client.
@@ -403,7 +512,14 @@ class ARROW_FLIGHT_SQL_EXPORT FlightSqlServerBase : public FlightServerBase {
   virtual arrow::Result<int64_t> DoPutCommandStatementUpdate(
       const ServerCallContext& context, const StatementUpdate& command);
 
-  /// \brief Create a prepared statement from given SQL statement.
+  /// \brief Execute an update Substrait plan.
+  /// \param[in] context  The call context.
+  /// \param[in] command  The StatementSubstraitPlan object containing the plan.
+  /// \return             The changed record count.
+  virtual arrow::Result<int64_t> DoPutCommandSubstraitPlan(
+      const ServerCallContext& context, const StatementSubstraitPlan& command);
+
+  /// \brief Create a prepared statement from a given SQL statement.
   /// \param[in] context  The call context.
   /// \param[in] request  The ActionCreatePreparedStatementRequest object containing the
   ///                     SQL statement.
@@ -413,6 +529,16 @@ class ARROW_FLIGHT_SQL_EXPORT FlightSqlServerBase : public FlightServerBase {
       const ServerCallContext& context,
       const ActionCreatePreparedStatementRequest& request);
 
+  /// \brief Create a prepared statement from a Substrait plan.
+  /// \param[in] context  The call context.
+  /// \param[in] request  The ActionCreatePreparedSubstraitPlanRequest object containing
+  ///                     the Substrait plan.
+  /// \return             A ActionCreatePreparedStatementResult containing the dataset
+  ///                     and parameter schemas and a handle for created statement.
+  virtual arrow::Result<ActionCreatePreparedStatementResult> CreatePreparedSubstraitPlan(
+      const ServerCallContext& context,
+      const ActionCreatePreparedSubstraitPlanRequest& request);
+
   /// \brief Close a prepared statement.
   /// \param[in] context  The call context.
   /// \param[in] request  The ActionClosePreparedStatementRequest object containing the
@@ -442,6 +568,39 @@ class ARROW_FLIGHT_SQL_EXPORT FlightSqlServerBase : public FlightServerBase {
       const ServerCallContext& context, const PreparedStatementUpdate& command,
       FlightMessageReader* reader);
 
+  /// \brief Begin a new transaction.
+  /// \param[in] context  The call context.
+  /// \param[in] request  Request parameters.
+  /// \return             The transaction ID.
+  virtual arrow::Result<ActionBeginTransactionResult> BeginTransaction(
+      const ServerCallContext& context, const ActionBeginTransactionRequest& request);
+
+  /// \brief Create a new savepoint.
+  /// \param[in] context  The call context.
+  /// \param[in] request  Request parameters.
+  /// \return             The savepoint ID.
+  virtual arrow::Result<ActionBeginSavepointResult> BeginSavepoint(
+      const ServerCallContext& context, const ActionBeginSavepointRequest& request);
+
+  /// \brief Release/rollback a savepoint.
+  /// \param[in] context  The call context.
+  /// \param[in] request  The savepoint.
+  virtual Status EndSavepoint(const ServerCallContext& context,
+                              const ActionEndSavepointRequest& request);
+
+  /// \brief Commit/rollback a transaction.
+  /// \param[in] context  The call context.
+  /// \param[in] request  The tranaction.
+  virtual Status EndTransaction(const ServerCallContext& context,
+                                const ActionEndTransactionRequest& request);
+
+  /// \brief Attempt to explicitly cancel a query.
+  /// \param[in] context  The call context.
+  /// \param[in] request  The query to cancel.
+  /// \return             The cancellation result.
+  virtual arrow::Result<CancelResult> CancelQuery(
+      const ServerCallContext& context, const ActionCancelQueryRequest& request);
+
   /// @}
 
   /// \name Utility methods
@@ -462,6 +621,9 @@ class ARROW_FLIGHT_SQL_EXPORT FlightSqlServerBase : public FlightServerBase {
   Status GetFlightInfo(const ServerCallContext& context, const FlightDescriptor& request,
                        std::unique_ptr<FlightInfo>* info) final;
 
+  Status GetSchema(const ServerCallContext& context, const FlightDescriptor& request,
+                   std::unique_ptr<SchemaResult>* schema) override;
+
   Status DoGet(const ServerCallContext& context, const Ticket& request,
                std::unique_ptr<FlightDataStream>* stream) final;
 
@@ -469,16 +631,46 @@ class ARROW_FLIGHT_SQL_EXPORT FlightSqlServerBase : public FlightServerBase {
                std::unique_ptr<FlightMessageReader> reader,
                std::unique_ptr<FlightMetadataWriter> writer) final;
 
+  const ActionType kBeginSavepointActionType =
+      ActionType{"BeginSavepoint",
+                 "Create a new savepoint.\n"
+                 "Request Message: ActionBeginSavepointRequest\n"
+                 "Response Message: ActionBeginSavepointResult"};
+  const ActionType kBeginTransactionActionType =
+      ActionType{"BeginTransaction",
+                 "Start a new transaction.\n"
+                 "Request Message: ActionBeginTransactionRequest\n"
+                 "Response Message: ActionBeginTransactionResult"};
   const ActionType kCreatePreparedStatementActionType =
       ActionType{"CreatePreparedStatement",
                  "Creates a reusable prepared statement resource on the server.\n"
                  "Request Message: ActionCreatePreparedStatementRequest\n"
                  "Response Message: ActionCreatePreparedStatementResult"};
+  const ActionType kCreatePreparedSubstraitPlanActionType =
+      ActionType{"CreatePreparedSubstraitPlan",
+                 "Creates a reusable prepared statement resource on the server.\n"
+                 "Request Message: ActionCreatePreparedSubstraitPlanRequest\n"
+                 "Response Message: ActionCreatePreparedStatementResult"};
+  const ActionType kCancelQueryActionType =
+      ActionType{"CancelQuery",
+                 "Explicitly cancel a running query.\n"
+                 "Request Message: ActionCancelQueryRequest\n"
+                 "Response Message: ActionCancelQueryResult"};
   const ActionType kClosePreparedStatementActionType =
       ActionType{"ClosePreparedStatement",
                  "Closes a reusable prepared statement resource on the server.\n"
                  "Request Message: ActionClosePreparedStatementRequest\n"
                  "Response Message: N/A"};
+  const ActionType kEndSavepointActionType =
+      ActionType{"EndSavepoint",
+                 "End a savepoint.\n"
+                 "Request Message: ActionEndSavepointRequest\n"
+                 "Response Message: N/A"};
+  const ActionType kEndTransactionActionType =
+      ActionType{"EndTransaction",
+                 "End a savepoint.\n"
+                 "Request Message: ActionEndTransactionRequest\n"
+                 "Response Message: N/A"};
 
   Status ListActions(const ServerCallContext& context,
                      std::vector<ActionType>* actions) final;
@@ -494,50 +686,50 @@ class ARROW_FLIGHT_SQL_EXPORT SqlSchema {
  public:
   /// \brief Get the Schema used on GetCatalogs response.
   /// \return The default schema template.
-  static std::shared_ptr<Schema> GetCatalogsSchema();
+  static const std::shared_ptr<Schema>& GetCatalogsSchema();
 
   /// \brief Get the Schema used on GetDbSchemas response.
   /// \return The default schema template.
-  static std::shared_ptr<Schema> GetDbSchemasSchema();
+  static const std::shared_ptr<Schema>& GetDbSchemasSchema();
 
   /// \brief Get the Schema used on GetTables response when included schema
   /// flags is set to false.
   /// \return The default schema template.
-  static std::shared_ptr<Schema> GetTablesSchema();
+  static const std::shared_ptr<Schema>& GetTablesSchema();
 
   /// \brief Get the Schema used on GetTables response when included schema
   /// flags is set to true.
   /// \return The default schema template.
-  static std::shared_ptr<Schema> GetTablesSchemaWithIncludedSchema();
+  static const std::shared_ptr<Schema>& GetTablesSchemaWithIncludedSchema();
 
   /// \brief Get the Schema used on GetTableTypes response.
   /// \return The default schema template.
-  static std::shared_ptr<Schema> GetTableTypesSchema();
+  static const std::shared_ptr<Schema>& GetTableTypesSchema();
 
   /// \brief Get the Schema used on GetPrimaryKeys response when included schema
   /// flags is set to true.
   /// \return The default schema template.
-  static std::shared_ptr<Schema> GetPrimaryKeysSchema();
+  static const std::shared_ptr<Schema>& GetPrimaryKeysSchema();
 
   /// \brief Get the Schema used on GetImportedKeys response.
   /// \return The default schema template.
-  static std::shared_ptr<Schema> GetExportedKeysSchema();
+  static const std::shared_ptr<Schema>& GetExportedKeysSchema();
 
   /// \brief Get the Schema used on GetImportedKeys response.
   /// \return The default schema template.
-  static std::shared_ptr<Schema> GetImportedKeysSchema();
+  static const std::shared_ptr<Schema>& GetImportedKeysSchema();
 
   /// \brief Get the Schema used on GetCrossReference response.
   /// \return The default schema template.
-  static std::shared_ptr<Schema> GetCrossReferenceSchema();
+  static const std::shared_ptr<Schema>& GetCrossReferenceSchema();
 
   /// \brief Get the Schema used on GetXdbcTypeInfo response.
   /// \return The default schema template.
-  static std::shared_ptr<Schema> GetXdbcTypeInfoSchema();
+  static const std::shared_ptr<Schema>& GetXdbcTypeInfoSchema();
 
   /// \brief Get the Schema used on GetSqlInfo response.
   /// \return The default schema template.
-  static std::shared_ptr<Schema> GetSqlInfoSchema();
+  static const std::shared_ptr<Schema>& GetSqlInfoSchema();
 };
 }  // namespace sql
 }  // namespace flight
